/* GLOBAL VARIABLE DECLARATIONS */
#include "defines.h";
#include <XBee.h>
#include <Servo.h>

// Array for receiving
int arguments[MAX_NUMBER_OF_ARGUMENTS];

XBee xbee = XBee();  //Create a new XBee object, attached to the Arduino.

//
XBeeAddress64 lampAdres = XBeeAddress64(0x0013A200, 0x40331975); // address of the Lamp_turret xBee, get this out of the xBee module with X-CTU application
XBeeAddress64 coAdres = XBeeAddress64(0x00000000, 0x00000000);  // address of the coordinator
// XBeeAddress64 sensAdres = XBeeAddress64(0x0013A200, 0x4033192E); //address of the sensor_turret, THIS IS ME!!


//
// flags and switches
boolean sendToCo=false;      // boolean for sending data to coordintor as well as to other turret
boolean servoTest=false;     // boolean for setting test servo modus, if false, then normal operation
boolean newSettings=true;    // flag indicating servo angles or LED brighness values were updated,
boolean demo_flag=false;     // flag indicating we are in demo mode 

int state = STATE_SCAN;      // see defines

//
// timing variables
int intervalA = 6;            // time between servoZ increments (minimum depends on how long it takes 
long timeA = 0;               // servoZ to make stepZ

int intervalB = 3000;         // time that the turret points to the brightes point found 
long timeB = 0;               // before continuing sweep

int intervalC=1000;          // time between sensor turret pointing and sending the angles on xBee to
int timeC=0;                 // Lamp_turret

int intervalD=100;          // used for a short delay between 2 consecutive xBee broadcasts (!?)
long timeD=0;

int scanNum = 0;                  // current scanning number
int initScanNum = 2;              // the default number of demo scans (when demo button is pressed)
int maxScanNum;                   // max number of scans in demo

// declaration for servo's
Servo servoZ;
Servo servoY;

int posZ = 0;      // position to write to servo
int minZ = 0;      // minimal angle; use in case there are obstacles or deviations in the servo sweep
int maxZ = 180;    // maximum angle
int midZ = 90;
int maxPosZ = 0;   // angle for maximum brightness detected
int dirZ = 1;      // direction
int prevDirZ = 0;  // previous direction, used for detecting when direction is turned
int stepZ = 1;     // stepsize

int posY = 0;      // position to write to servo
int minY = 0;      // minimal angle; use in case there are obstacles or deviations in the servo sweep
int maxY = 180;    // maximum angle
int midY = 90;
int maxPosY = 0;   // angle for maximum brightness detected
int dirY = 1;      // direction
int prevDirY = 0;  // previous direction, used for detecting when direction is turned
int stepY = 180/20;// stepsize

// pin and variable declaration for LED's
int statusLed = 13;  // pin assignment for status Led
int debugLed = 12;   // pin assignment for debug Led
int redPin = 6;      // pin assignment red led(-driver) PWM output
int greenPin = 5;    // pin assignment green led(-driver) PWM output
int bluePin = 3;     // pin assignment blue led(-driver) PWM output

int redVal = 0;      // variables for brightness values of R,G and B led's
int greenVal = 0;
int blueVal = 0;

// pin and variable declaration for light sensor
int LDRpin = 1;
int LDRVal = 0;
int prevLDRVal = 0;

// pin and variable declaration for button
int buttonPin = 2;
int buttonState = 0;
int prevButtonState = 0;
int state_toggle = 0;

long lastDebounceTime = 0;   // the last time the output pin was toggled
long debounceDelay = 100;    // the debounce time; increase if the output flickers
long lastToggleTime = 0;     // 
int toggleDelay = 500;       // interval in which no toggl eis possible (calmer toggle behavior)


///////////
/* SETUP */
///////////
void setup () 
{
  xbee.begin(115200);

  pinMode(redPin, OUTPUT);
  pinMode(greenPin, OUTPUT);
  pinMode(bluePin, OUTPUT);
  pinMode(statusLed, OUTPUT);
  pinMode(LDRpin, INPUT);
  pinMode(debugLed, OUTPUT);

  servoY.attach(10);
  servoZ.attach(11);


  analogWrite(redPin, redVal );
  analogWrite(greenPin, greenVal);
  analogWrite(bluePin, blueVal );
  digitalWrite(statusLed, LOW);
  digitalWrite(debugLed, LOW);

  midZ=(minZ+maxZ)/2;
  midY=(minY+maxY)/2;
  posZ=midZ;
  posY=midY;

  maxScanNum=initScanNum;

  if (servoTest==true){
    state=100;
  }

  timeA=millis();
  timeB=millis();
  timeC=millis();
  timeD=millis();
}

///////////////
/* MAIN LOOP */
///////////////
void loop()
{



  // check for incoming XBee communication
  checkXBee();
  
  // check if the demo button is pressed
  checkButton();
  
  digitalWrite(debugLed, state_toggle); // show the demo-status on a LED


  // handle changed brightnesses or angles
  if (newSettings==true){
    servoZ.write(posZ);
    servoY.write(posY);
    analogWrite(redPin, redVal);
    analogWrite(greenPin,greenVal);
    analogWrite(bluePin, blueVal);  
    newSettings=false; 
  }



  // state machine ///////////////
  switch (state){
    // 0
  case STATE_STANDBY:
    // do nothing here
    digitalWrite(statusLed, HIGH);// turn ON the standby LED
    break;

    // 1  
  case STATE_SCAN:        // searching highest light value
    digitalWrite(statusLed, LOW); // turn OFF the standby LED
    checkLDR();

    // sweep through complete half sphere
    //
    // every intervalA do: 
    if (millis() >= (timeA + intervalA )){

      // prepare led outputs correlating to LDR value
      redVal = LDRVal/4;
      greenVal = LDRVal/4;
      blueVal = LDRVal/4;

      // every time servo Z changes direction (see below), 
      if (prevDirZ != dirZ){
        // sweep servoY from minimum to maximum and back with stepY increments
        if (dirY){
          posY= posY+stepY;
          if (posY>maxY){
            posY=maxY;
            dirY=0;
            state=STATE_SCAN_DONE;
          }
        }
        else{
          posY=posY-stepY;
          if (posY<minY){
            posY=minY;
            dirY=1;
            state=STATE_SCAN_DONE;
          }
        }     
        prevDirZ = dirZ;
        newSettings=true; 
      }

      // sweep servoZ from minimum to maximum and back with stepZ increments
      if (dirZ){
        posZ= posZ+stepZ;
        if (posZ>maxZ){
          posZ=maxZ;
          dirZ=0;
        }
      }
      else{
        posZ=posZ-stepZ;
        if (posZ<minZ){
          posZ=minZ;
          dirZ=1;
        }
      }       
      newSettings=true; 
      timeA=millis();
    }
    break;

//
  case STATE_SCAN_DONE:
    // prepare to go to position of highest light value
    posZ=maxPosZ;
    posY=maxPosY; 

    // prepare for state STATE_POINTING
    scanNum++;    
    state=STATE_POINTING_1;
    timeB= millis();
    break;

//
  case STATE_POINTING_1:      // wait intervalB milliseconds 
    // increment brightness of red led to maximum over the time of intervalB

    if (millis()<=timeB+intervalC){      
      // if intervalC has not yet passed
      // increase red brightness in linear relation to time passed of intervalC
      redVal=map((millis()-timeB),0,intervalC,0,255);
      greenVal=0;
      blueVal=0;
      newSettings=true;       // signal to execute new settings

      addArgument(redVal);
      addArgument(greenVal);
      addArgument(blueVal);
      sendData(lampAdres,FUNCTION_SET_COLOR);  
          
      // if send to coordinator is enabled    
      if (sendToCo){
      addArgument(redVal);
      addArgument(greenVal);
      addArgument(blueVal);
      sendData(coAdres,FUNCTION_SET_COLOR);
      }
      
    }
    else{
      // intervalC has passed, go to next step
      timeD=millis();
      state=STATE_POINTING_2;
    }

    break;

//
  case STATE_POINTING_2:
    // little delay after last xBee command
    if (millis()>=timeD+intervalD){  
      // after delay, send the position of max brightness to Lamp_turret
  //    posZ=maxPosZ;
  //    posY=maxPosY;
      redVal=prevLDRVal/4;
      greenVal=prevLDRVal/4;
      blueVal=prevLDRVal/4;
      newSettings=true;       // signal to execute new settings     
      
      addArgument(posZ);
      addArgument(posY);
      addArgument(prevLDRVal/4);
      addArgument(prevLDRVal/4);
      addArgument(prevLDRVal/4);
      sendData(lampAdres,FUNCTION_SET_ANGLE_AND_COLOR); 
          
      // if send to coordinator is enabled    
      if (sendToCo){
      addArgument(posZ);
      addArgument(posY);
      addArgument(prevLDRVal/4);
      addArgument(prevLDRVal/4);
      addArgument(prevLDRVal/4);
      sendData(coAdres,FUNCTION_SET_ANGLE_AND_COLOR); 
      }

      state=STATE_POINTING_3;
    }

    break;

//
  case STATE_POINTING_3:
    if(millis()>=timeB+intervalB){
      // if intervalB has passed

      state = STATE_SCAN;   // set state to continue scanning
      prevLDRVal=0;         // reset stored max LDRValue

      // in demo mode, only maxScanNum scans may be made before the sensor_turret enters standby again.
      // check # of demo scans
      if(scanNum>=maxScanNum && demo_flag==true){ //
        // when maxScanNum done, then
        state=STATE_RESET;
      }
      //   
    }

    break;

//    
    case STATE_RESET:
            // set servo's to midpoint
        posZ= midZ;
        posY= midY;
        // set RGB LED's
        redVal= 0;
        greenVal= 0;
        blueVal= 50;

        newSettings=true;       // signal main loop to execute new settings  

        // prepare the positions and RGB leds for sending over xBee
        addArgument(posZ);
        addArgument(posY);
        addArgument(redVal);
        addArgument(greenVal);
        addArgument(blueVal);
        // send data (the 5 argumentss) to function number 5 on the lamp_turret
        sendData(lampAdres,FUNCTION_SET_ANGLE_AND_COLOR);
        
      // if send to coordinator is enabled
      if (sendToCo){
      addArgument(posZ);
      addArgument(posY);
      addArgument(redVal);
      addArgument(greenVal);
      addArgument(blueVal);
      sendData(coAdres,FUNCTION_SET_ANGLE_AND_COLOR); 
      }

        demo_flag=false;
        state_toggle=false;
        

        state=STATE_STANDBY;      // enter standby
    break;

//
  case STATE_COPY_DELAY:
    if (millis()>timeC+intervalC){
      state = STATE_SCAN;   // continue scanning
    }
    break;




  case 100:                        // for testing servo angles (blocking while executed!)
    if (servoTest==true){
      testServos();
      servoTest=false; // signal that servo test has been done  
    }
    break;
  }
}























